# -*- coding: utf-8 -*-
"""
    vault-dump-restore
    ~~~~~~~~~~~~~~~~~~

    Dumps keys from a Hashicorp Vault instance to be able to restore it on
    another instance.
"""
__author__ = 'Hugo cisneiros'
__version__ = "0.4.1"
